/*
 * File: nwsim.c
 * Author: Russell J Clark
 * Date: October 7, 1996
 *
 * This is the network simulator module. It receives packets from the
 * sender and forwards them on to the receiver. This communication
 * is performed using standard input and standard output.
 *
 * This simulator will "corrupt" the data according to the bit-error
 * rate given.
 *
 */

#include <stdio.h>
#include <values.h>
#include <sys/types.h>
#include <time.h>

#include "nwsim.h"

#define ERR	-1
#define OK	1


/* Global variables */

#define MAX_RANDOM	32767
/*
 * Return a random number between 0 and max inclusive. 
 */
int RandomInt(max)
int max;
{ 
  int i, rand();
 
  if (max == 0)
     return(0);
 
  i = rand();

  i = i % (max + 1);

  return(i);
 
} /* RandomInt */
/*
 * Randomly change bits of a data segment according to the
 * given bit error rate.
 *
 * Return the total number of bits changed.
 */
int ErrorizePacket(data,length,bitErrorRate)
unsigned char *data;
int length;
float bitErrorRate;
{  
    register int byte, bitcount;
    register unsigned char bit;
    register unsigned char *dataptr = data;
    register int bitsChanged = 0;

    for (byte=0; byte < length; byte++)
    {
    	bit = 1;
        for (bitcount=0; bitcount<8; bitcount++)
        {
	    if (RandomInt(MAX_RANDOM) < (int)((float)MAX_RANDOM * bitErrorRate))
	    {
		/* debug statements 
    		fprintf(stderr,"ErrorizePacket: reversing bit %d of byte %d\n",
    				bitcount,byte);
    		fprintf(stderr,"ErrorizePacket: changing from %x \n",*dataptr);
   		*/ 

               	*dataptr =  (*dataptr & ~bit) + (~(*dataptr & bit) & bit);

		/* debug statements 
		fprintf(stderr,"ErrorizePacket: changing to %x \n",*dataptr);
		*/

		bitsChanged++;
	    }
	    bit = bit << 1;
	}
	dataptr++;
    }
    return(bitsChanged);
} /* ErrorizePacket */


void PrintUsage(command)
char *command;
{
	fprintf(stderr,"Usage: %s [-p bit_error_rate] \n",command);
	exit(1);
} /* PrintUsage */

main(argc,argv)
int argc;
char *argv[];
{
    unsigned char messageData[PACKET_TOTAL_SIZE];
    int   bitsChanged;
    int   bytesRead, bytesSent;
    float bitErrorRate;


    if (argc > 3) PrintUsage(argv[0]);

    if ((argc == 3) && (strncmp(argv[1],"-p",2) == 0))
    {
	sscanf(argv[2],"%f",&bitErrorRate);
    }

    fprintf(stderr,"%s: Using bit error rate of  %f\n",argv[0],bitErrorRate);

    /*
     * Seed the random number generator with the current time.
     */
    srand((int)time(NULL));

    /*
     * Continuously listen for messages from the sender and forward them
     * on to the receiver.
     */

    /* while ((messageData = getchar()) != EOF) */
    while ((bytesRead = fread(messageData,1,PACKET_TOTAL_SIZE,stdin)) > 0)
    {

	bitsChanged = ErrorizePacket(messageData,bytesRead,bitErrorRate);

	fprintf(stderr,"%s: %d out of %d bits changed\n",argv[0],
	bitsChanged,8*bytesRead);

	if ((bytesSent = fwrite(messageData,1,PACKET_TOTAL_SIZE,stdout))
		     != bytesRead )
	    fprintf(stderr,"%s: warning, fwrite only sent %d of %d bytes\n",
			argv[0],bytesSent,bytesRead);

    }

} /* main */
