/*
 * File: sender.c
 * Author: Russell J Clark
 * Date: October 10, 1996
 * 
 * This is the sender portion of the nwsim system. This program creates
 * a data packet and sends it to the simulator using standard output.
 * The data is generated randomly or read from a data file.
 */

#include <fcntl.h>
#include <stdio.h>
#include <sys/types.h>

#include "nwsim.h"

main(argc,argv)
int argc;
char *argv[];
{
    int bytesRead, bytesSent;
    unsigned char messageData[PACKET_TOTAL_SIZE];
    unsigned char errorCheckData[PACKET_TRAILER_SIZE];
    FILE *datafile;

    if (argc < 2)
    {
    	fprintf(stderr,"Usage: %s data_file \n",argv[0]);
    	exit(1);
    }

    /*
     * Build the message to be sent to the Network Simulator
     */

    if  ((datafile = fopen(argv[1],"r")) < 0 ) {
    	printf("%s: failed to open data file %s\n",argv[0],argv[1]);
    	exit();
    }

    if  ((bytesRead = fread(messageData,1,PACKET_DATA_SIZE,datafile)) <
    		PACKET_DATA_SIZE )
    {
    	fprintf(stderr,"%s: failed to read %d bytes from file %s\n",argv[0],
    			PACKET_DATA_SIZE,argv[1]);
    	fprintf(stderr,"%s: only %d bytes read \n",argv[0],bytesRead);
    	exit();
    } else {

    	/*
    	 * Calculate the error detection code.
    	 */
    	errorCheckData[0] = CalculateEvenParity(messageData,bytesRead);

    	fprintf(stderr,"%s: Parity is %d\n",argv[0],errorCheckData[0]);

    	/*
    	 * Copy the error detection code onto the end of the data buffer.
    	 */
    	memcpy(&messageData[PACKET_DATA_SIZE],errorCheckData,
    			PACKET_TRAILER_SIZE);

    	if ((bytesSent = fwrite(messageData,1,PACKET_TOTAL_SIZE,stdout))
    			             != PACKET_TOTAL_SIZE )
    		fprintf(stderr,"%s: warning, fwrite only sent %d of %d bytes\n",
    			   argv[0],bytesSent,bytesRead);
    	else
    		fprintf(stderr,"%s: message of size %d bytes sent.\n",argv[0],bytesSent);
    } /* else */


    close(datafile);

} /* main */

